# HW_827.py: liefert Graph zur Pulsmessung
# 19.04.2025

# Vergleich:
# Messung mit HW-827: 75 bpm
# Messung mit Breuer-Puls-Oximeter (angegebene Genauigkeit: 2%): zwischen 76 und 77 bpm (nach längerem Einpendeln)

# Wenn der Gaph vollständig ist, wird die Messung angehalten.
# Nach einer Pause von 4 s wird eine erneute Messung vorgenommen.
# Während einer solchen Pause kann das Programm mit dem Taster T0 (oben links) angehalten werden.
# Mit Hilfe der Skala (1 s) kann man dann die Puls-Zeit bestimmen.


from time import sleep, sleep_ms, sleep_us, ticks_ms, ticks_diff
from sys import exit
from machine import Pin, ADC, SPI
import vga1_8x16 as font1 
import vga1_bold_16x32 as font2  
import st7789

# Taster zum Abbruch des Programms
taster = Pin(0, Pin.IN, Pin.PULL_UP)

# display instanziieren und konfigurieren...
spi = SPI(1, baudrate=20000000, polarity=1, sck=Pin(18), mosi=Pin(19))
display = st7789.ST7789(spi, 135, 240,  reset=Pin(23, Pin.OUT), cs=Pin(5, Pin.OUT), dc=Pin(16, Pin.OUT), backlight=Pin(4, Pin.OUT), rotation=3) # Landscape
display.init()
bg = st7789.color565(200,200,255)

# puls instanziieren und konfigurieren; puls.read() liefert gemessene Spannung zwischen 0 (0 V) und 4095 (3,6 V)...
puls = ADC(Pin(36))  # ADC0
puls.atten(ADC.ATTN_11DB)  # Messbereich: 0 - 3.6 V;
puls.width(ADC.WIDTH_12BIT) # 12 Bit: Messwerte von 0 bis 4095

# Vorbereitung (global_max und global_min bestimmen)...
display.fill(bg)
display.text(font1, 'Puls messen mit HW-827', 10, 10, st7789.BLACK, bg)
sleep(2)
display.text(font1, 'Min-Max-Suche: ', 15, 35, st7789.BLACK, bg)

delta_t0 = 25 # Zeit in ms zwischen zwei aufeinander folgende Messungen (in ms)

# Maximum und Minimum bestimmen...
display.text(font1, 'Min. und Max. bestimmen...', 15, 35, st7789.BLACK, bg)
glob_min = 4095
glob_max = 0
for i in range(220):
  puls_wert = puls.read()
  display.pixel(10+i, 60, st7789.BLUE) # Fortschrittsbalken 
  # print(puls_wert)
  if puls_wert < glob_min:
    glob_min = puls_wert
  if puls_wert > glob_max:
    glob_max = puls_wert
  sleep_ms(delta_t0)
diff_5 = int((glob_max - glob_min) / 5)
# Anzeige-Bereich für die Werte etwas vergrößern...
glob_min = int(glob_min - diff_5)
glob_max = int(glob_max + diff_5)
glob_diff = glob_max - glob_min
# zur Kontrolle...
print('glob_min =', glob_min, ', glob_max =', glob_max) 
print('glob_diff =', glob_diff)

while True:
  display.fill(bg)
  display.text(font1, 'Puls messen mit HW-827', 10, 10, st7789.BLACK, bg)  
  # Messreihe mit 220 Messpunkten (delta_t = 50 ms) aufzeichnen und graphisch darstellen...
  display.rect(9,30,222,100,st7789.BLUE)   # Rahmen fuer Bargraph mit Skala...
  # t0 = ticks_us() # nur zu Testzwecken
  for i in range(220):
    puls_wert = puls.read()
    # print(puls_wert) # zu Testzwecken
    # bargraph in WE...
    puls_wert_bar = int((puls_wert - glob_min)  / glob_diff * 100) # wenn puls_wert = glob_max, dann puls_wert_bar = 100 
    # print(puls_wert_bar)
    if puls_wert_bar > 0:
      display.line(10+i, 120, 10+i, 120-puls_wert_bar, st7789.RED)
      if i % 20 == 0:
        display.line(10+i, 30, 10+i, 125, st7789.BLUE) # Skala: 20 Pixel entsprechen 0,5 s.
      sleep_ms(delta_t0 - 1) # Ein einziger Schleifendurchlauf benötigt dann ziemlich genau 25 ms.
  # t1 = ticks_ms() # zu Testzwecken
  # print('Zeit: ', ticks_diff(t1, t0)) # zu Testzwecken (zeigt ca. 11028 ms an)

  # Wenn innerhalb der nächsten 4 s der Taster T0 betätigt wird,
  # wird das Programm abgebrochen und das Diagramm kann manuell ausgewertet werden.
  # Ansonsten wird eine neue Messreihe aufgenommen...
  t0 = ticks_ms()
  wait = True
  while wait:
    t1 = ticks_ms()
    wait = ticks_diff(t1, t0) < 4000
    if taster.value() == 0: # falls Taster T0 betätigt wird...
      exit() # Programm beenden



  
    
      
  
  
  




  









